#!/bin/bash
###########################################################
# convert_functions.sh
# defaults and function library for vdr image plugin 
#
# History:
# 2004-07-26  a.brachold - change mkdir -v -p to mkdir -p, for work with busybox
# 2004-07-07  a.brachold - prevent zerosize vdr-image.pnm if pbmmake fail
# 2004-06-27  a.brachold - remove anymore necessary parts for create black.mpg
# 2004-06-27  a.brachold - remove ImageMagick stuff
# 2004-06-22  a.brachold - cleanup logging 
# 2004-06-01  a.brachold - remove unused f_get_vdrsetup
#                          - add check_requirement
# 16/03/2004  initial implementation, split off of convert_jump.sh
###########################################################
[ "z$DEBUG" = "zyes" ] && set -xv

#
# set Defaults here
#

#for the lousy concept of splitting normal software and system software
PATH=/usr/local/bin:$PATH

# use pnm-cache for zooming, this works well on slow, single cpu machines
USE_ZOOMCACHE=true

# Set AR is'nt set from the vdr prefs
# TODO: get reports, does it work ?
ASPECT_RATIO="${ASPECT_RATIO:-"4:3"}"
#ASPECT_RATIO="${ASPECT_RATIO:-"16:9"}"

# set to true to include picture numbers in jump menu when using netpbm
SHOW_NUMBERS="${SHOW_NUMBERS:-"false"}"

#
#
# NO CONFIGURATION REQUIRED BELOW
#
#

#
# logging function, used to create debugging output
#
function logit() {
   #uncomment this for syslog logging
   #logger "convert.sh: $*"                                                                            
   echo "$*" >> "$CONVERT_LOGFILE"
   echo "$*"
}
# end logit

#
# set reasonable defaults, check directory structure
#
convert_init() {
CONVERT_TEMPDIR="$1"
CALLED="$2"

   CONVERT_LOGFILE="$CONVERT_TEMPDIR/convert.log"
   echo "" > "$CONVERT_LOGFILE"

   check_requirements

   [ ! -d "$CONVERT_TEMPDIR" ] && mkdir -p "$CONVERT_TEMPDIR" | tee -a "$CONVERT_LOGFILE"
   
   # create default Images for background and 3x3 grid
   BACKGROUND="$CONVERT_TEMPDIR/vdr-image.pnm"
   [ ! -s "$BACKGROUND" ] && pbmmake -black 704 576 > "$BACKGROUND"
   [ ! -s "$BACKGROUND" ] && logit "Error $BACKGROUND does not exist,\n it should be a 704x576 black pnm file" && exit 1

   DUMMY_TILE="$CONVERT_TEMPDIR/vdr-tile.pnm"
   [ ! -s "$DUMMY_TILE" ] && pbmmake -black 256 192 > "$DUMMY_TILE"
   [ ! -s "$DUMMY_TILE" ] && logit "Error $DUMMY_TILE does not exist,\n it should be a 256x192 black pnm file" && exit 1

   # defaults for FLIP, ORIGINAL and ROTATE
   ROTATE="NO"
   ORIGINAL="NO"
   FLIP="cat"

   # log conversion start
   DATE=`date +%d.%m.%y-%T`
   logit "convert_init: $0: $DATE startet ..."
   logit "convert_init: $CALLED"
}

function check_requirement() {
		type "$1" > /dev/null 2>&1
		found=$?
		[ $found -ne 0 ] && logit "Error ! External command \"$1\" not found ! Please adjust PATH or install it inside \"$PATH\"" && exit 1
}
#
# check requirement external programs
#
function check_requirements() {
REQUIREMENTS="file \
  sed \
  cut \
  pbmmake \
  mpeg2enc \
  ppmntsc \
  ppmtoy4m \
  pnmflip \
  pnmcomp \
  pnmscale \
  pnmpaste \
  ppmlabel \
  pnmcat \
  pnmfile \
  pnmcut"

  for i in $REQUIREMENTS
	do 
		check_requirement "$i"
	done
}

#
# identify netpbm tool required to convert the image
#
function getpnmtool() { 
IMAGE="$1"

   FILE_DETECT=$(file -i -L -b "$IMAGE" 2>/dev/null)
   FILE_TYPE=$(echo "$FILE_DETECT"|cut -f1 -d"/")
   FILE_SUB_TYPE=$(echo "$FILE_DETECT"|cut -f2 -d"/")
  
   # preparation for mpeg2 still display capapility
   # funny thing, no zoom/jump menu will be available for these
   if [ $FILE_DETECT = "video/mpeg" ] ; then
      logit "getpnmtool: Detected $FILE_DETECT for $IMAGE, returning"
      return
   fi 

   case "$FILE_TYPE" in
   image)
      logit  "getpnmtool: Image file type $FILE_TYPE/$FILE_SUB_TYPE detected for $IMAGE"
      ;;
   *)
      logit "getpnmtool: Not an image file : $IMAGE"
      exit 1
      ;;
   esac

   case "$FILE_SUB_TYPE" in
   jpeg)
		check_requirement jpegtopnm
	    TO_PNM="jpegtopnm -quiet"
      ;;
   tiff)
		check_requirement tifftopnm
	    TO_PNM="tifftopnm -quiet"
      ;;
   bmp)
		check_requirement bmptoppm
		TO_PNM="bmptoppm -quiet"
      ;;
   x-bmp)
		check_requirement bmptoppm
		TO_PNM="bmptoppm -quiet"
      ;;
   png)
		check_requirement pngtopnm
		TO_PNM="pngtopnm -quiet"
      ;;
   x-png)
		check_requirement pngtopnm
		TO_PNM="pngtopnm -quiet"
      ;;
   x-portable-pixmap*)
		check_requirement cat
        TO_PNM="cat"
      ;;
   x-portable-graymap*)
		check_requirement cat
        TO_PNM="cat"
      ;;
   gif)
		check_requirement giftopnm
        TO_PNM="giftopnm -quiet"
      ;;
   *)
      logit "getpnmtool: Filetype $FILE_TYPE/$FILE_SUB_TYPE unsupported , EXIT 1"
      exit 1
      ;;
   esac
   logit "getpnmtool: using $TO_PNM to convert $IMAGE"
}
# end getpnmtool

#
# return resolution of image file in X_RES_TMP and Y_RES_TMP
#
function getresolution() {
IMAGE="$1"

   getpnmtool "$IMAGE"

    TEMP2=`echo $($TO_PNM "$IMAGE" | pnmfile -)`
  # tested with netpbm 10.0, Parse pnmfile output "-: PPM raw, 768 by 576  maxval 255"
    X_RES_TMP=$(echo -e "$TEMP2"| cut -d " " -f 4)
    Y_RES_TMP=$(echo -e "$TEMP2"| cut -d " " -f 6)
}

#
# convert_imagetompeg
# take $1 and transform it to a 704 576 mpeg2 still image
#
function convert_imagetompeg() {
IMAGE="$1"
MPEG="$2"
FLIP="$3"

   if test "$ORIGINAL" == "YES"
    then
      logit "Orignal"
   fi
   if test "$ROTATE" == "YES"
   then
      logit "Flip Image"
      X_RES="$Y_RES_TMP"
      Y_RES="$X_RES_TMP"
   else
      logit "don't Flip Image"
      X_RES="$X_RES_TMP"
      Y_RES="$Y_RES_TMP"
    fi
    logit "X-RES: $X_RES | Y-RES: $Y_RES"

   if [ $ASPECT_RATIO = "16:9" ]
    then 
	    MPEG2ENC_AR=3
	    SCALE_MIN_ASP=163
      SCALE_MAX_ASP=178
    else 
	    MPEG2ENC_AR=2
	    SCALE_MIN_ASP=125
	    SCALE_MAX_ASP=133
   fi 

# uh, ah no scaling done at all, wont work in all cases
#   # hardcoded, replaces old AR calculations
#   # TODO: make NTSC aware
#   SCALE="pnmscale -xysize 704 576"
#   BG_GEN="pnmcomp -align=center -valign=middle - $BACKGROUND"
#

# ok aspect calc's
#calc for the asp corrector, if less than 1.25 it will be scaled to have _apropiate_ black bars
# until now it only worked for 4/3 (3/4) photos to rotate. 
# also support for 16/9 is implemented here
OUT_DISPLAY_Y=576 #480 for ntsc...
OUT_DISPLAY_X=704 #??? ntsc ???

    if [ "$((${X_RES}00 / ${Y_RES}))" -lt $SCALE_MIN_ASP ]
        then
        X_CALC=$((${OUT_DISPLAY_Y}000 / $Y_RES * $X_RES / 1000))
        SCALE="pnmscale -xysize $X_CALC ${OUT_DISPLAY_Y}"
        BG_GEN="pnmcomp -align=center -valign=middle - $BACKGROUND"
    elif [ "$((${X_RES}00 / ${Y_RES}))" -gt $SCALE_MAX_ASP ]
        then
        Y_CALC=$((${OUT_DISPLAY_X}000 / $X_RES * $Y_RES / 1000))
        SCALE="pnmscale -xysize ${OUT_DISPLAY_X} $Y_CALC"
        BG_GEN="pnmpaste /dev/stdin 0 $(((${OUT_DISPLAY_Y} - $Y_CALC) / 2)) $BACKGROUND"
    else
        SCALE="pnmscale -xysize ${OUT_DISPLAY_X} ${OUT_DISPLAY_Y}"
        BG_GEN="pnmcomp -align=center -valign=middle - $BACKGROUND"
    fi


   getpnmtool "$IMAGE"

   logit "convert_imagetompeg: converting $IMAGE to $MPEG"
   logit "convert_imagetompeg: TO_PNM: $TO_PNM"
   logit "convert_imagetompeg: FLIP:   $FLIP"
#   logit "convert_imagetompeg: SCALE:  $SCALE"
#   logit "convert_imagetompeg: BG_GEN: $BG_GEN"
   logit "convert_imagetompeg: ASPECT: $ASPECT_RATIO"
 
   

   $TO_PNM "$IMAGE" | $FLIP | $SCALE | $BG_GEN | \
   ppmntsc --pal | ppmtoy4m -v 0 -F 25:1 -A $ASPECT_RATIO    | \
   mpeg2enc -v 0 -f 7 -T 90 -F 3 -np -a $MPEG2ENC_AR -o "$MPEG" | \
   tee -a "$CONVERT_LOGFILE"
}
